#ifndef _QUADTREE_H
#define _QUADTREE_H
#include <image.h>
/*
 * Structure contenant les informations ncessaires  chaque noeud du quadtree
 */
typedef struct {
    unsigned char uniforme : 1; 
    unsigned char erreur   : 2;
    unsigned char moyenne;
    // 3 bits + 1 octet < 2 octets => cette structure occupe 2 octets en mmoire
} qt_element;

/*
 * Quadtree
 * L'arbre est stock sous forme de tableau pour un parcours  haute vitesse
 * et un gain de mmoire certain par l'absence de pointeurs
 * 
 * tree[0] est la racine.
 * 
 * pour un noeud i :
 *  tree[i/4-(i%4)?0:1] est son pre.
 *  tree[i*4+1] est son premier fils.
 *  tree[i*4+4] est son dernier fils.
 * 
 * Le calcul est peu plus complexe pour le pre car la racine n'occupe qu'un
 * seul lment de l'arbre alors que tous les autres tages de l'arbre sont
 * des multiples de 4. Il faut donc dcaller d'une valeur si on veut le pre
 * du 4e fils.
 *
 * Grace  cette structure, on sait que toutes les feuilles de l'arbre sont
 *  partir de taille/4.
 *
 * Le quadtree tant un arbre complet, sa taille est calcule ds le dpart
 * et l'arbre n'est allou qu'en une unique allocation.
 */
typedef struct {
    qt_element * tree;
    long int taille;/* taille mmoire du quadtree */
    char * name; /* Nom de l'image associe  ce quadtree */
    unsigned char colors;
    unsigned char puissance; /* puissance de 2 indiquant la largeur de l'image*/
} quadtree;

/* Gnre le quadtree  partir d'une image lu et stocke 
 * dans la structure image renvoit un pointeur sur un 
 * quadtree ou NULL si erreur
 * Les images non carrs et dont la largeur et la hauteur ne sont pas des
 * puissances de 2 ne sont pas traits
 * et la programme retourne NULL;
 * */

quadtree * quadtree_create(image * img, int pertes);

/*
 * Transforme un quadtree en image pgm
 */
image * quadtree_to_image(quadtree * q, int niveau);

/*
 * Libre la mmoire occupe par un quadtree
 */
void quadtree_free(quadtree *);

#endif
