#include <fbab.h>

/*
 * Ouvre le fichier et initialise la lecture/criture
 */
FBAB * fbab_open(char * filename, char * flags)
{
    FBAB * f = NULL;

    f = malloc(sizeof(FBAB));
    if ( f ) {
	f->file = fopen(filename,flags);
	if ( f->file == NULL ) {
	    free(f);
	    return NULL;
	}
	f->tampon = 0;
	f->position = 7;
    }
    
    return f;
}

/*
 * Force l'criture du tampon dans le fichier et ferme le fichier
 */
void fbab_close(FBAB * f)
{
    if ( f->position != 7 )
	fwrite(&(f->tampon),1,1,f->file);

    fclose(f->file);
    free(f);
}

/*
 * Force l'criture du tampon dans le fichier
 */
void fbab_flush(FBAB * f)
{
    if ( f->position != 7 ) {
	fwrite(&(f->tampon),1,1,f->file);
	f->position = 7;
	f->tampon = 0;
    }
}

/*
 * crit un bit dans le fichier f.
 * Retourne 1 pour une criture russie et 0 dans le cas contraire.
 */
inline int fbab_writebit(unsigned char bit, FBAB * f)
{
    if ( bit )
	f->tampon |= (1<<f->position);

    f->position--;

    if ( f->position == -1 ) {
	if ( fwrite(& f->tampon,sizeof(char),1,f->file) != 1 ) {
	    perror("fwrite");
	    return 0;
	}
	f->tampon = 0;
	f->position = 7;
    }

    return 1;
}

/*
 * crit 8 bits dans f
 * Retourne 1 pour une criture russie et 0 dans le cas contraire.
 */
int fbab_write8(unsigned char data, FBAB * f)
{
    int i;

    for ( i=7 ; i != -1 ; i-- )
	if ( ! fbab_writebit(data & (1<<i), f) )
	    return 0;
    
    return 1;
}

/*
 * crit un entier compris entre 0 et 510 
 * selon la mthode du codage diffrentiel donne
 * Retourne le nombre de bits cods.
 */
int fbab_writediff(unsigned short data, FBAB * f)
{
    assert(data < 511);
    int i;

    if ( data < 4 ) {
	// Prfixe 0
	if ( ! fbab_writebit(0, f) )
	    return 0;
	// Code sur 2 bits
	if ( ! fbab_writebit(data & 2,f) )
	    return 0;
	if ( ! fbab_writebit(data & 1,f) )
	    return 0;
	return 3;
    } else if ( data < 20 ) {
	// Prfixe 10
	if ( ! fbab_writebit(1, f) )
	    return 0;
	if ( ! fbab_writebit(0, f) )
	    return 0;
	// Code sur 4 bits
	data -= 4;
	for ( i=3 ; i > -1 ; i-- )
	    if ( ! fbab_writebit(data & (1<<i), f) )
		return 0;
	return 6;
    } else if ( data < 52 ) {
	// Prfixe 110 
	if ( ! fbab_writebit(1, f) )
	    return 0;
	if ( ! fbab_writebit(1, f) )
	    return 0;
	if ( ! fbab_writebit(0, f) )
	    return 0;
	// Code sur 5 bits
	data -= 20;
	for ( i=4 ; i > -1 ; i-- )
	    if ( ! fbab_writebit(data & (1<<i), f) )
		return 0;
	return 8;
    } else {
	// Prfixe 111
	if ( ! fbab_writebit(1, f) )
	    return 0;
	if ( ! fbab_writebit(1, f) )
	    return 0;
	if ( ! fbab_writebit(1, f) )
	    return 0;
	// Code sur 9 bits
	data -= 52;
	for ( i=8 ; i > -1 ; i-- )
	    if ( ! fbab_writebit((data & (1<<i))?1:0, f) )
		return 0;
	return 12;
    }
}

/*
 * Lit un bit dans f et le place dans bit
 * Retourne 1 si la lecture fut possible, 0 sinon.
 */
inline int fbab_readbit(unsigned char * bit, FBAB * f)
{
    if ( f->position == 7 )
	if ( fread(&f->tampon, 1, 1, f->file) != 1 ) {
	    if ( ! feof(f->file) ) {
		perror("fread");
		return 0;
	    }
	}

    *bit = (f->tampon & (1<<f->position)) ? 1 : 0;
    f->position--;

    if ( f->position == -1 )
	f->position = 7;

    return 1;
}

/*
 * Lit 8 bits de f
 * Retourne 1 si la lecture fut possible, 0 sinon.
 */
int fbab_read8(unsigned char * data, FBAB * f)
{
    int i;
    unsigned char bit;

    *data = 0;
    for ( i=7 ; i >= 0 ; i-- ) {
	if ( ! fbab_readbit(&bit, f) )
	    return 0;
	if ( bit )
	    *data |= (1<<i);
    }

    return 1;
}

/*
 * Lit un entier entre 0 et 510
 * selon la mthode donne pour le codage diffrientiel
 */
int fbab_readdiff(unsigned short * data, FBAB * f)
{
    unsigned char bit;
    int i;

    *data = 0;

    fbab_readbit(&bit, f);
    if ( bit == 0 ) {
	// 2 bits  lire
	if ( ! fbab_readbit(&bit, f) )
	    return 0;
	*data |= (bit<<1);
	if ( ! fbab_readbit(&bit, f) )
	    return 0;
	*data |= bit;
	return 1;
    }

    fbab_readbit(&bit, f);
    if ( bit == 0 ) {
	// 4 bits  lire
	for ( i=3 ; i > -1 ; i-- ) {
	    if ( ! fbab_readbit(&bit, f) )
		return 0;
	    *data |= (bit<<i);
	}
	*data += 4;
	return 1;
    }

    fbab_readbit(&bit, f);
    if ( bit == 0 ) {
	// 5 bits  lire
	for ( i=4 ; i > -1 ; i-- ) {
	    if ( ! fbab_readbit(&bit, f) )
		return 0;
	    *data |= (bit<<i);
	}
	*data += 20;
	return 1;
    }

    for ( i=8 ; i > -1 ; i-- ) {
	// 9 bits  lire
	if ( ! fbab_readbit(&bit, f) )
	    return 0;
	*data |= (bit<<i);
    }
    *data += 52;

    return 1;
}

