#include <main.h>

void usage(char * binaryname)
{
    fprintf(stderr,"Usage : %s [-p %%] [-D] [-n N] -c|-d filename\n",
	    binaryname);
    fprintf(stderr,"  -p %% :  la compression, pourcentage de pertes sur"
	    " l'image compresse. [0-100]\n");
    fprintf(stderr,"  -D   :  la compression, active le codage diffrentiel."
	    "\n");
    fprintf(stderr,"  -n N :  la dcompression, niveau de dcompression."
	    " [>0]\n");
    fprintf(stderr,"  -d   : dcompresse le fichier filename qui doit tre un "
	    "fichier au format qtc.\n");
    fprintf(stderr,"  -c   : compresse le fichier filename qui doit tre un "
	    "fichier au format pgm\n");
    fprintf(stderr,"         en niveau de gris.\n");
}

int options(int argc, char ** argv, 
	int * pertes, int * niveau, int * diff, int * action, char ** imagename)
{
    signed char c;
    
    *action = 0;
    *diff = 0;
    *niveau = -1;
    *pertes = 0;
    
    while ( (c = getopt(argc, argv, "cdp:Dn:")) != -1 ) {
	switch(c) {
	    case 'c':
		*action = 'c';
		break;
	    case 'd':
		*action = 'd';
		break;
	    case 'p':
		*pertes = strtol(optarg, NULL, 10);
		if ( *pertes > 100 || *pertes < 0 ) {
		    usage(argv[0]);
		    return 0;
		}
		break;
	    case 'D':
		*diff = 1;
		break;
	    case 'n':
		*niveau = strtol(optarg, NULL, 10);
		if ( *niveau < 0 ) {
		    usage(argv[0]);
		    return 0;
		}
		break;
	    default:
		usage(argv[0]);
		return 0;
	}
    }

    if ( optind == argc ) {
	usage(argv[0]);
	return 0;
    }

    *imagename = malloc(strlen(argv[optind])+1);
    if ( *imagename == NULL ) {
	fprintf(stderr,"Erreur fatale, impossible de stocker le nom de "
		"l'image\n");
	return 0;
    }
    strcpy(*imagename, argv[optind]);

    return 1;
}

int main(int argc, char **argv)
{
    int pertes, niveau, diff, action;
    char * imagename = NULL;
    image * img = NULL;
    quadtree * q = NULL;

    if ( ! options(argc, argv, &pertes, &niveau, &diff, &action, &imagename) )
	return 1;

    switch(action) {
	case 'c': // Codage 
	    if ( ! (img = pgm_load(imagename)) ) {
		fprintf(stderr,"Impossible de lire %s\n",imagename);
		return 2;
	    }

	    if ( ! (q = quadtree_create(img, pertes)) ) {
		fprintf(stderr,"Compression impossible\n");
		return 3;
	    }

	    if ( ! codec_code(q,diff) ) {
		fprintf(stderr,"Codage impossible\n");
		return 4;
	    }

	    image_free(img);
	    quadtree_free(q);
	    free(imagename);
	    return 0;

	case 'd': // Dcodage
	    if ( ! (q = codec_decode(imagename)) ) {
		fprintf(stderr,"Dcodage impossible\n");
		return 2;
	    }

	    if ( ! (img = quadtree_to_image(q, niveau)) ) {
		fprintf(stderr,"Dcompression impossible\n");
		return 3;
	    }

	    if ( ! (pgm_write(img)) ) {
		fprintf(stderr,"criture impossible\n");
		return 4;
	    }

	    image_free(img);
	    quadtree_free(q);
	    free(imagename);
	    return 0;
	default:
	    usage(argv[0]);
    }

    return 1;
}
