#include <pgm.h>

#define ZAPBLANKS() while ( isspace(c = fgetc(file)) ) ; ungetc(c,file)

image * pgm_load(char * filename)
{
    assert(filename != NULL);

    FILE * file = NULL;
    image * pgm = NULL;
    pgm_type t;
    int c,i;
    char * temp = NULL;

    // Allocate & initialize some memory
    pgm = malloc(sizeof(image));
    if ( pgm == NULL ) {
	perror("malloc");
	return NULL;
    }
    pgm->data = NULL;
    /*
    i = strlen(filename)-4; // on enlve "pgm"
    pgm->name = malloc(i+1);
    if ( pgm->name == NULL ) {
	free(pgm);
	return NULL;
    }
    strncpy(pgm->name, filename, i);
    pgm->name[i] = 0;
    */
    temp = strdup(filename);
    temp[strlen(temp)-4] = 0; // On retire .pgm
    pgm->name = strdup(basename(temp));
    free(temp);
    
    // Open PGM
    file = fopen(filename, "r");
    if ( file == NULL ) {
	perror("fopen");
	free(pgm->name);
	free(pgm);
	return NULL;
    }

    // Check magic number
    c = fgetc(file);
    if ( c != 'P' ) {
	fprintf(stderr,"PGM Load : Incorrect magic number\n");
	free(pgm->name);
	free(pgm);
	fclose(file);
	return NULL;
    }

    // File type
    c = fgetc(file);
    switch(c) {
	case '2':
	    t = ASCII;
	    break;
	case '5':
	    t = RAW;
	    break;
	default:
	    fprintf(stderr,"PGM Load : Incorrect file type\n");
	    free(pgm->name);
	    free(pgm);
	    fclose(file);
	    return NULL;
    }

    ZAPBLANKS();

    // Zap comments
    do {
	c = fgetc(file);
	if ( c == '#' ) {
	    while ( (c = fgetc(file)) != '\n' )
		;
	} else {
	    if ( ungetc(c,file) != c ) {
		perror("ungetc");
		free(pgm->name);
		free(pgm);
		fclose(file);
		return NULL;
	    }
	    c = 0;
	}
    } while ( c != 0 );

    // Informations
    fscanf(file, "%d", &(pgm->width)); 
    fscanf(file, "%d", &(pgm->height));
    fscanf(file, "%d", &(pgm->colors));

    ZAPBLANKS();

    // Allocate memory for image data
    pgm->data = calloc(pgm->width * pgm->height, sizeof(unsigned char));
    if ( pgm->data == NULL ) {
	free(pgm);
	fclose(file);
	return NULL;
    }

    // Data
    switch(t) {
	case RAW:
	    if ( fread(pgm->data, sizeof(char), pgm->width * pgm->height, file)
		    < pgm->width * pgm->height )
		printf("Attention, %s tronqu\n",filename);
	    break;
	case ASCII:
	    for ( i=0 ; i < pgm->width * pgm->height ; i++ ) {
		if ( fscanf(file,"%hhu",&(pgm->data[i])) != 1 ) {
		    printf("Attention, %s tronqu\n",filename);
		    break;
		}
	    }
	    break;
    }

    fclose(file);

    return pgm;
}

int pgm_write(image * img) 
{
    assert(img != NULL);

    FILE * file = NULL;
    char * filename = NULL;
    time_t t = time(NULL);

    filename = malloc(strlen(img->name)+9);
    strcpy(filename, "pgm/");
    strcat(filename, img->name);
    strcat(filename, ".pgm");

    file = fopen(filename,"w");
    if ( file == NULL ) {
	perror("fopen");
	return 0;
    }

    // Header
    fprintf(file,"P5\n#%s%d %d %d\n",
	    ctime(&t),
	    img->width, img->height, img->colors);

    // Data
    if ( fwrite(img->data, sizeof(unsigned char), img->width*img->height, file)
	    != img->width * img->height) {
	perror("fwrite");
	return 0;
    }

    fclose(file);
    free(filename);

    return 1;
}
