#include "quadtree.h"
#include <stdio.h>
#include <string.h>
#include "image.h"


/* Met les valeurs dans le quadtree 
 * l largeur tailleinit correspond
 * a la case a partie de laquelle l'image
 * est stocke dans le quadtree*/
void quadtree_Charge(quadtree *q, image *img,int tailleinit, int i, int j, int l)
{   
    static int pos=0;
    
    if (pos == 0)
	pos = tailleinit;
    if(pos >= q->taille)
    { 
	pos = 0;
	return;
    }
    if(l == 1)
    {
    	(*q).tree[pos].moyenne=img->data[i*img->width+j];
	/* on met l'uniformit  1 pour permettre le calcul
	 * de l'uniformit des peres*/
	(*q).tree[pos].uniforme=1;
    	pos++;
    	return ;
    }
    /* on divise la largeur de l'image par 2 jusuq'a avoir une largeur de 1
     * pixel a chaque etape les indices sont calculs lors du rappel de la 
     * fonction*/
    //l = l/2;
    l >>= 1;
    quadtree_Charge(q,img,tailleinit, i   , j   ,l);
    quadtree_Charge(q,img,tailleinit, i   , j+l ,l);
    quadtree_Charge(q,img,tailleinit, i+l , j+l ,l);
    quadtree_Charge(q,img,tailleinit, i+l , j   ,l);	
}


/* Creation du quadtree
 * pertes = pourcentage de pertes  la compression
 * */
quadtree* quadtree_create(image *img, int pertes)
{
    quadtree* quad=NULL;
    int i=1,j=0,k=0,count=0,tmp=0;
    int profondeur=0;
    float diff1=0,diff2=0,diff3=0,diff4=0;

    static int size=sizeof(int)*8;
    /* Verification que l'image est carre */
    if(img->height != img->width)
    {
	fprintf(stderr,"L'image n'est pas une image carre\n");
	return NULL;
    }
    /* verification que la largeur de l'image est une puissance de 2*/
    tmp=img->width;
    for(k=0,count=0;k<size;k++)
    {
	if(tmp & (1<<k))
	    count++;
	if(count > 1)
	{
	    fprintf(stderr,"La largeur de l'image n'est pas une puissance de 2\n");
	    return NULL;
	}
    }
    /* initialisation du quadtree*/
    quad=(quadtree*)malloc(sizeof(quadtree));
    quad->name=NULL;
    quad->colors=img->colors;
    quad->puissance=0;
	
    /* Calcul de la taille du quadtree et de sa profondeur
     * la profondeur correspond a la puissance de 2 de l'image*/    
    quad->taille=1;
    while((img->width & i) == 0)
    {
	i=i<<1;
	j+=2;
	quad->taille=quad->taille+(1<<j);
	profondeur++;
    }    
    quad->puissance=profondeur;
    quad->tree=(qt_element*)malloc(sizeof(qt_element)*quad->taille);
    /* nom de l'image */
    quad->name=(char*)malloc(sizeof(char)*strlen(img->name)+1);
    strcpy(quad->name,img->name);
    /* Remplissage du quadtree */
    quadtree_Charge(quad,img,(quad->taille-(img->width*img->height)),0,0,img->width);

    /* initialisation de i une case avant celle ou commence le stokage de l'image
     * */
    //i=quad->taille-1-(img->width*img->height);    
    i = (quad->taille>>2)-1; // i>>2 == i / 4
    while(i>=0)
    {
	// i << 2 == i * 4
	quad->tree[i].moyenne = 
	    (quad->tree[(i<<2)+1].moyenne
	     +quad->tree[(i<<2)+2].moyenne
	     +quad->tree[(i<<2)+3].moyenne
	     +quad->tree[(i<<2)+4].moyenne)
	    /4;
	
	/* calcul des differences par rapport a la moyenne */
	diff1=quad->tree[i].moyenne-quad->tree[(i<<2)+1].moyenne;
	diff2=quad->tree[i].moyenne-quad->tree[(i<<2)+2].moyenne;
	diff3=quad->tree[i].moyenne-quad->tree[(i<<2)+3].moyenne;
        diff4=quad->tree[i].moyenne-quad->tree[(i<<2)+4].moyenne;
	
	/* verification que les moyennes sont dans la marge des pertes
	 * pour detecter l'uniformit */
	if(((diff1>=0 && diff1<=pertes)||(diff1<0 && diff1 >= -pertes))
	&&((diff2>=0 && diff2<=pertes)||(diff2<0 && diff2 >= -pertes))
	&&((diff3>=0 && diff3<=pertes)||(diff3<0 && diff1 >= -pertes))
	&&((diff4>=0 && diff4<=pertes)||(diff4<0 && diff1 >= -pertes)))
	{
	    /* un pere est uniforme si ses fils le sont aussi
	     * */
	    if((quad->tree[(i<<2)+1].uniforme == 1)
		&&(quad->tree[(i<<2)+2].uniforme == 1)
		&&(quad->tree[(i<<2)+3].uniforme == 1)
		&&(quad->tree[(i<<2)+4].uniforme == 1))
	    {
		quad->tree[i].uniforme=1;
		quad->tree[i].erreur=0;
	    }
	    else
	    {
		quad->tree[i].uniforme=0;

		quad->tree[i].erreur=
		(quad->tree[(i<<2)+1].moyenne
		+quad->tree[(i<<2)+2].moyenne
		+quad->tree[(i<<2)+3].moyenne
		 +quad->tree[(i<<2)+4].moyenne)
		&3;
		/*&3 == %4*/
	    }
	}
	else
	{
	    quad->tree[i].uniforme=0;

	    quad->tree[i].erreur=
		(quad->tree[(i<<2)+1].moyenne
		+quad->tree[(i<<2)+2].moyenne
		+quad->tree[(i<<2)+3].moyenne
		 +quad->tree[(i<<2)+4].moyenne)
		&3;
		/*&3 == %4*/
	}
	i--;
    }
  
   return quad;
}


/* Met les valeurs du quadtree dans l'image
 * l largeur i et j sont les indices des cases pour le parcours
 * taillinit correspond a la premiere case ou l'image est commanc a tre stock
 * dans le quadtree*/
void quadtree_decompImage(quadtree *q, image *img,int tailleinit, int i, int j, int l)
{   
    static int pos=0;
    
    if (pos == 0)
	pos = tailleinit;
    if(pos >= q->taille)
    { 
	pos = 0;
	return;
    }
    if(l == 1)
    {
    	img->data[i*img->width+j]=(*q).tree[pos].moyenne;
    	pos++;
    	return ;
    }
    l = l/2;
    quadtree_decompImage(q,img,tailleinit, i   , j   ,l);
    quadtree_decompImage(q,img,tailleinit, i   , j+l ,l);
    quadtree_decompImage(q,img,tailleinit, i+l , j+l ,l);
    quadtree_decompImage(q,img,tailleinit, i+l , j   ,l);	
}

/* Remplit l'image  partir d'un certain niveau
 * */
static void niveau_progressif(quadtree* q, int niveau, int nivact, int i)
{
    if((((i<<2))+4)>=q->taille)
	    return;
    if(nivact >= niveau)
    {
	    q->tree[(i<<2)+1].moyenne=q->tree[i].moyenne;
	    q->tree[(i<<2)+2].moyenne=q->tree[i].moyenne;
	    q->tree[(i<<2)+3].moyenne=q->tree[i].moyenne;
	    q->tree[(i<<2)+4].moyenne=q->tree[i].moyenne;
    }
    niveau_progressif(q,niveau,nivact+1,(i<<2)+1);
    niveau_progressif(q,niveau,nivact+1,(i<<2)+2);
    niveau_progressif(q,niveau,nivact+1,(i<<2)+3);
    niveau_progressif(q,niveau,nivact+1,(i<<2)+4);
}


/* Converti un quadtri en image 
 * niveau = niveau de dcompression
 * */
image* quadtree_to_image(quadtree *q, int niveau)
{
    image* img=NULL;

    if(q == NULL)
    {
	fprintf(stderr,"Erreur lors de la creation du qaudtree\n");
	return NULL;
    }
    /* allocation de l'image */    
    if((img=(image*)malloc(sizeof(image))) == NULL)
	return NULL;
    /* nom de l'image */
    if(q->name != NULL)
    {	
	img->name=(char*)malloc(sizeof(char)*strlen(q->name)+1);
        strcpy(img->name,q->name);
    }
    img->colors=q->colors;
    /* Taille de l'image */
    img->width=1<<q->puissance;
    img->height=1<<q->puissance;
    img->data=(unsigned char*)malloc(sizeof(unsigned char)*(img->width*img->height));

    /* Modification du quadtree selon le niveau demand */
    if((niveau >= 0)&&(niveau < q->puissance))
   	niveau_progressif(q,niveau,0,0);
    
    /* les valeurs du quadtree sont stockes dans l'image */
    quadtree_decompImage(q,img,q->taille-(img->height*img->width),0,0,img->width);

    return img;
}

/* Libere la mmoire du quadtree
 * */
void quadtree_free(quadtree* q)
{
    if(q == NULL)
	    return;
    /* nom de l'image */
    free(q->name);
    q->name=NULL;    
    free(q->tree);
    free(q);    
}
